# coding: utf-8
from __future__ import absolute_import
from __future__ import unicode_literals
from __future__ import print_function

from django.conf import settings
from django.utils.module_loading import import_string
from django.utils.deconstruct import deconstructible


class Proxy(object):
    def __init__(self, backend):
        self.__dict__['_backend_string_'] = backend
        self.__dict__['_initialized_'] = False
        self.__dict__['_upper_'] = None

    def _prepare(self):
        if self.__dict__['_initialized_']:
            return
        self.__dict__['_initialized_'] = True
        self.__dict__['_upper_'] = import_string(self.__dict__['_backend_string_'])()

    def __getattr__(self, key):
        if '_constructor_args' == key:
            return self.__dict__[key]
        self._prepare()
        return getattr(self.__dict__['_upper_'], key)

    def __setattr__(self, key, value):
        if '_constructor_args' == key:
            self.__dict__[key] = value
            return
        self._prepare()
        return setattr(self.__dict__['_upper_'], key, value)


@deconstructible
class CeleryTaskFileStorage(Proxy):
    def __init__(self):
        super(CeleryTaskFileStorage, self).__init__(
            settings.CELERY_TASK_FILE_STORAGE if hasattr(settings, 'CELERY_TASK_FILE_STORAGE')
            else settings.DEFAULT_FILE_STORAGE)



# -*- coding: utf-8 -*-
# Generated by Django 1.11.3 on 2017-07-12 10:45
from __future__ import unicode_literals

from django.db import migrations, models
import heavy_celery.storage


class Migration(migrations.Migration):

    dependencies = [
        ('heavy_celery', '0003_celerytasklog'),
    ]

    operations = [
        migrations.AlterField(
            model_name='celerytask',
            name='result_file',
            field=models.FileField(blank=True, null=True, storage=heavy_celery.storage.CeleryTaskFileStorage(), upload_to=b'', verbose_name='\u7d50\u679c\u30d5\u30a1\u30a4\u30eb'),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.3 on 2017-07-12 03:00
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('heavy_celery', '0002_worker_workertasklog'),
    ]

    operations = [
        migrations.CreateModel(
            name='CeleryTaskLog',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('level', models.IntegerField(choices=[(0, 'debug'), (1, 'info'), (2, 'warning'), (3, 'error'), (4, 'critical'), (5, 'exception')])),
                ('text', models.TextField()),
                ('task', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='heavy_celery.CeleryTask')),
            ],
        ),
    ]



# coding: utf-8
from __future__ import absolute_import
from __future__ import unicode_literals
from __future__ import print_function
import threading
from datetime import datetime

# from django.conf import settings
from django.utils import timezone
# from django.utils.module_loading import import_string
from croniter import croniter


_app = None
_task = {}
_user = {}
_middleware = None


def set_task(task):
    _task[threading.current_thread()] = task
    _user[threading.current_thread()] = task.user


def reset_task():
    _task.pop(threading.current_thread(), None)
    _user.pop(threading.current_thread(), None)


def get_user():
    global _middleware

    user = _user.get(threading.current_thread(), None)
    if user:
        return user

    if _middleware is None:
        try:
            from crequest.middleware import CrequestMiddleware
        except ImportError:
            class CrequestMiddleware(object):
                @classmethod
                def get_request(klass):
                    return None
        _middleware = CrequestMiddleware

    request = _middleware.get_request()
    if request:
        return request.user
    return None


def get_now():
    return timezone.localtime(timezone.now()).replace(second=0, microsecond=0)


def get_next_cron(cron_expr, now=None):
    if now is None:
        now = get_now()
    return croniter(cron_expr, now).get_next(datetime)


class Logger(object):
    class F(object):
        def __init__(self, func, level_id):
            self.func = func
            self.level_id = level_id

        def __call__(self, *args, **kwargs):
            return self.func(self.level_id, *args, **kwargs)

    def __init__(self, task, root_logger, propagate):
        from .models import CeleryTaskLog
        self.root_logger = root_logger
        self.propagate = propagate
        self.task = task
        self.klass = CeleryTaskLog
        self._map = dict(self.klass.LEVEL_CHOICES)

        for level_id, level_string in self.klass.LEVEL_CHOICES:
            setattr(self, level_string, self.F(self._log, level_id))
            setattr(self, 'get_{}'.format(level_string), self.F(self._get_log, level_id))

    def _log(self, level, text):
        if self.propagate:
            getattr(self.root_logger, self._map[level])(text)
        self.klass.objects.create(task=self.task, level=level, text=text)

    def _get_log(self, level):
        return self.klass.objects.create(task=self.task, level=level, text="")


def get_logger(default_logger, propagate=True):
    t = _task.get(threading.current_thread(), None)
    if t:
        return Logger(t, default_logger, propagate)
    return default_logger
